@@ -122,7 +122,8 @@ def run_espei(run_settings):
     if generate_parameters_settings is not None:
         refdata = generate_parameters_settings['ref_state']
         excess_model = generate_parameters_settings['excess_model']
-        dbf = generate_parameters(phase_models, datasets, refdata, excess_model)
+        ridge_alpha = generate_parameters_settings['ridge_alpha']
+        dbf = generate_parameters(phase_models, datasets, refdata, excess_model, ridge_alpha=ridge_alpha)
         dbf.to_file(output_settings['output_db'], if_exists='overwrite')
 
     if mcmc_settings is not None:
@@ -145,7 +146,7 @@ def run_espei(run_settings):
                 logging.warning("The number of cores chosen is larger than available. "
                                 "Defaulting to run on the {} available cores.".format(cores))
             # TODO: make dask-scheduler-verbosity a YAML input so that users can debug. Should have the same log levels as verbosity
-            scheduler = LocalCluster(n_workers=cores, threads_per_worker=1, processes=True)
+            scheduler = LocalCluster(n_workers=cores, threads_per_worker=1, processes=True, memory_limit=0)
             client = ImmediateClient(scheduler)
             client.run(logging.basicConfig, level=verbosity[output_settings['verbosity']])
             logging.info("Running with dask scheduler: %s [%s cores]" % (scheduler, sum(client.ncores().values())))
@@ -174,7 +175,7 @@ def run_espei(run_settings):
         else:
             restart_trace = None
 
-        # load the remaning mcmc fitting parameters
+        # load the remaining mcmc fitting parameters
         iterations = mcmc_settings.get('iterations')
         save_interval = mcmc_settings.get('save_interval')
         chains_per_parameter = mcmc_settings.get('chains_per_parameter')
@@ -197,6 +198,7 @@ def run_espei(run_settings):
         return dbf, sampler
     return dbf
 
+
 def main():
     """
     Handle starting ESPEI from the command line.
@@ -238,5 +240,6 @@ def main():
 
     run_espei(input_settings)
 
+
 if __name__ == '__main__':
     main()
