@@ -4,7 +4,6 @@ Plotting of input data and calculated database quantities
 import warnings
 from collections import OrderedDict
 
-from cycler import cycler
 import matplotlib.pyplot as plt
 import matplotlib.lines as mlines
 import numpy as np
@@ -15,7 +14,8 @@ from pycalphad.plot.eqplot import eqplot, _map_coord_to_variable, unpack_conditi
 
 from espei.utils import bib_marker_map
 from espei.core_utils import get_data, get_samples, list_to_tuple, \
-    endmembers_from_interaction, build_sitefractions, ravel_zpf_values, ravel_conditions
+    ravel_zpf_values
+from espei.utils import endmembers_from_interaction, build_sitefractions
 
 plot_mapping = {
     'T': 'Temperature (K)',
@@ -72,6 +72,7 @@ def plot_parameters(dbf, comps, phase_name, configuration, symmetry, datasets=No
     mix_plots = [ ('Z', 'HM_MIX'), ('Z', 'SM_MIX')]
     comps = sorted(comps)
     mod = Model(dbf, comps, phase_name)
+    mod.models['idmix'] = 0
     # This is for computing properties of formation
     mod_norefstate = Model(dbf, comps, phase_name, parameters={'GHSER'+(c.upper()*2)[:2]: 0 for c in comps})
     # Is this an interaction parameter or endmember?
